#region Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

using Meebo.Properties;
using System.Configuration;

#endregion

namespace Meebo
{
    public partial class MeeboBrowser : Form
    {

        #region Variables

        GraphicsPath splashPath;
        Pen splashPen;

        #endregion

        #region Private Methods

        public MeeboBrowser()
        {
            InitializeComponent();
            webBrowser1.ObjectForScripting = this;
            splashPath = CreateRoundedPath(splashPanel.Size, 5);
            splashPen = new Pen(Color.DarkGray, 2);
            splashPanel.Region = new Region(splashPath);
        }

        private void Meebo_Load(object sender, EventArgs e)
        {
            Location = Point.Empty;
            Size = Screen.PrimaryScreen.WorkingArea.Size;
            BackColor = Color.FromArgb(255, 1, 1, 1);
            TransparencyKey = Color.White;

            splashBrowserLabel.Text += typeof(MeeboBrowser).Assembly.GetName().Version.ToString(2);

            try
            {
                webBrowser1.Navigate(Settings.Default.HomeUri);
                webBrowser1.Visible = true;
                focusTimer.Enabled = true;
            }
            catch (System.Runtime.InteropServices.COMException ex)
            {
                if (ex.ErrorCode == 0x00040153 - 0x7fffffff)
                {
                    MessageBox.Show("Could not load rendering engine", Application.ProductName + " v" + typeof(MeeboBrowser).Assembly.GetName().Version.ToString(2), MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    Application.Exit();
                    return;
                }
                throw;
            }
        }

        private void Meebo_Closing(object sender, CancelEventArgs e)
        {
            Settings.Default.Save();
        }

        private GraphicsPath CreateRoundedPath(Size size, int radius)
        {
            GraphicsPath path = new GraphicsPath();
            path.AddArc(0, 0, radius * 2, radius * 2, 180, 90);
            path.AddLine(radius, 0, size.Width - radius, 0);
            path.AddArc(size.Width - radius * 2, 0, radius * 2, radius * 2, 270, 90);
            path.AddLine(size.Width, radius, size.Width, size.Height - radius);
            path.AddArc(size.Width - radius * 2, size.Height - radius * 2, radius * 2, radius * 2, 0, 90);
            path.AddLine(size.Width - radius, size.Height, radius, size.Height);
            path.AddArc(0, size.Height - radius * 2, radius * 2, radius * 2, 90, 90);
            path.AddLine(0, size.Height - radius, 0, radius);
            return path;
        }

        private void splashPanel_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawPath(splashPen, splashPath);
        }

        private void Meebo_Deactivate(object sender, EventArgs e)
        {
            focusTimer.Enabled = true;
        }

        private void focusTimer_Tick(object sender, EventArgs e)
        {
            focusTimer.Enabled = false;
        }

        private object execScript(string script)
        {
            if (webBrowser1.Document == null) return null;
            return webBrowser1.Document.InvokeScript("execScript", new object[] { script, "javascript" });
        }

        private void webBrowser1_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            if (e.Url.OriginalString == "about:blank") return;
            execScript(@"if (document.MeeboBrowserScriptUri == null) {
                document.MeeboBrowserScriptUri = '" + Settings.Default.ScriptUri.Replace("'", @"\'") + @"';
                (function(scriptURL) {
                    var scriptElem = document.createElement('SCRIPT');
                    scriptElem.setAttribute('language', 'JavaScript');
                    scriptElem.setAttribute('src', scriptURL);
                    document.body.appendChild(scriptElem);
                })(document.MeeboBrowserScriptUri + 'meebobrowser.js?' + (new Date()).getTime());
            }");
        }

        private void webBrowser1_DocumentTitleChanged(object sender, EventArgs e)
        {
            execScript(@"if (typeof(MeeboBrowser) != 'undefined') {
                MeeboBrowser.DocumentTitleChanged();
            };");
        }

        private void SetBuddyListVisibility(bool? state)
        {
            execScript(@"if (typeof(MeeboBrowser) != 'undefined') {
                MeeboBrowser.SetBuddyListVisibility(" + state.ToString().ToLower() + @");
            };");
        }

        private void notifyIcon1_Click(object sender, EventArgs e)
        {
            MouseEventArgs me = e as MouseEventArgs;
            if ((me != null) && (me.Button == MouseButtons.Left))
            {
                if (focusTimer.Enabled)
                {
                    SetBuddyListVisibility(null);
                }
                else
                {
                    if (TopMost)
                    {
                        SetBuddyListVisibility(null);
                    }
                    else
                    {
                        SetBuddyListVisibility(true);
                    }
                    Show();
                    Activate();
                }
            }
        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            MouseEventArgs me = e as MouseEventArgs;
            if ((me != null) && (me.Button == MouseButtons.Left))
            {
                if (focusTimer.Enabled)
                {
                    Hide();
                }
                else
                {
                    Show();
                    Activate();
                }
            }
        }

        private void notifyIcon1_BalloonTipClicked(object sender, EventArgs e)
        {
            Show();
            Activate();
        }

        private void menuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = ((ToolStripMenuItem)sender);
            execScript(@"if (typeof(MeeboBrowser) != 'undefined') {
                MeeboBrowser.MenuItem_" + menuItem.Name + "_Click(" + (!menuItem.Checked).ToString().ToLower() + @");
            };");
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Meebo_Closing(sender, null);
            Application.Exit();
        }

        private void EnsureSettingProperty(string key)
        {
            if (Settings.Default.Properties[key] == null)
            {
                SettingsProperty property = new SettingsProperty(key);
                property.PropertyType = typeof(string);
                property.Provider = Settings.Default.Properties["HomeUri"].Provider;
                property.Attributes.Add(typeof(UserScopedSettingAttribute), new UserScopedSettingAttribute());
                Settings.Default.Properties.Add(property);
            }
        }

        #endregion

        #region Public Methods

        public bool AddMenuItem(string key, string text)
        {
            if ((key == null) || 
                (key.Length == 0) ||
                (text == null) ||
                (text.Length == 0) ||
                contextMenuStrip1.Items.ContainsKey(key)) return false;
            ToolStripItem menuItem;
            if (text.Trim('-').Length == 0)
            {
                menuItem = new ToolStripSeparator();
                menuItem.Name = key;
            }
            else
            {
                menuItem = new ToolStripMenuItem(text, null, menuItem_Click, key);
            }
            contextMenuStrip1.Items.Insert(contextMenuStrip1.Items.Count - 1, menuItem);
            return true;
        }

        public bool SetMenuItemChecked(string key, bool state)
        {
            if (!contextMenuStrip1.Items.ContainsKey(key)) return false;
            ((ToolStripMenuItem)contextMenuStrip1.Items[key]).Checked = state;
            return true;
        }

        public bool RemoveMenuItem(string key)
        {
            if (contextMenuStrip1.Items.ContainsKey(key) && (key != exitToolStripMenuItem.Name))
            {
                contextMenuStrip1.Items.RemoveByKey(key);
                return true;
            }
            return false;
        }

        public int ClearMenuItems()
        {
            try
            {
                return contextMenuStrip1.Items.Count - 1;
            }
            finally
            {
                while (contextMenuStrip1.Items.Count > 1)
                    contextMenuStrip1.Items.RemoveAt(0);
            }
        }

        public string GetSetting(string key)
        {
            EnsureSettingProperty(key);
            object value = Settings.Default[key];
            if (value != null) return value.ToString();
            return null;
        }

        public void SetSetting(string key, string value)
        {
            EnsureSettingProperty(key);
            Settings.Default[key] = Convert.ChangeType(value, Settings.Default.Properties[key].PropertyType);
        }

        public string TransparentColor
        {
            get
            {
                return ColorTranslator.ToHtml(base.TransparencyKey);
            }
            set
            {
                base.TransparencyKey = ColorTranslator.FromHtml(value);
            }
        }

        public bool SplashPanelVisible
        {
            get
            {
                return splashPanel.Visible;
            }
            set
            {
                splashPanel.Visible = value;
            }
        }

        public string NotifyIconText
        {
            get
            {
                return notifyIcon1.Text;
            }
            set
            {
                notifyIcon1.Text = value;
            }
        }

        public void ShowBalloonTip(int timeout, string tipTitle, string tipText, string toolTipIcon)
        {
            notifyIcon1.ShowBalloonTip(timeout, tipTitle, tipText, (ToolTipIcon)Enum.Parse(typeof(ToolTipIcon), toolTipIcon, true));
        }

        #endregion

    }
}
